﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Text;
using System.Windows.Forms;
using JMS.DVB;
using JMS.DVB.EPG;
using JMS.DVB.EPG.Descriptors;
using JMS.DVB.EPG.Tables;
using JMS.DVB.TS;
using SatIp.Guide.Rtp;

namespace SatIp.Guide
{
    public partial class MainForm : Form
    {
        private static RunTimeLoader _runtimeLoader = RunTimeLoader.Instance;
        
        private Parser EITParser = new Parser(null);
        private Uri _streamUri;
        private RtpClient _client;
        private Dictionary<string, bool> _entries = new Dictionary<string, bool>();
        private List<EpgEntry> _listItems = new List<EpgEntry>();
        
        public MainForm()
        {
            InitializeComponent();
            _client = new RtpClient();            
            EITParser.SectionFound += EITSectionFound;
        }

        

        public int SortIndex = 2;        

        private void EITSectionFound(Section section)
        {
            if (((section != null) && section.IsValid) && !ProcessEITEvents(section.Table as EIT))
            {
                ProcessCitEvents(section.Table as CITPremiere);
            }
        }

        private bool ProcessCitEvents(CITPremiere citPremiere)
        {
            if (citPremiere == null)
            {
                return false;
            }
            if (citPremiere.IsValid)
            {
                foreach (var descriptor in citPremiere.Descriptors)
                {
                    var entry = descriptor as ContentTransmissionPremiere;
                    if ((entry != null) && entry.IsValid)
                    {
                        AddEntry(entry);
                    }
                }
            }
            return true;
        }

        private bool ProcessEITEvents(EIT eit)
        {
            if (eit == null)
            {
                return false;
            }
            if (eit.IsValid)
            {
                foreach (var entry in eit.Entries)
                {
                    if ((EventStatus.Running == entry.Status))
                    {
                        AddEntry(eit.ServiceIdentifier, entry);
                    }
                }
            }
            return true;
        }

        private void AddEntry(ContentTransmissionPremiere entry)
        {
            CITPremiere table = (CITPremiere)entry.Table;
            string name = null;
            string description = null;
            bool flag = false;
            foreach (DateTime time in entry.StartTimes)
            {
                string key = string.Format("{0}-{1}", entry.ServiceIdentifier, time.Ticks);
                if (this._entries.ContainsKey(key))
                {
                    break;
                }
                this._entries[key] = true;
                if (!flag)
                {
                    flag = true;
                    LoadEventData(out name, out description, table.Descriptors);
                }
                this._listItems.Add(new EpgEntry(entry.ServiceIdentifier, name, description, time.ToLocalTime(), table.Duration));
            }
        }

        private void AddEntry(ushort serviceIdentifier, EventEntry entry)
        {
            string key = string.Format("{0}-{1}", serviceIdentifier, entry.EventIdentifier);
            if (!this._entries.ContainsKey(key))
            {
                string str2;
                string str3;
                this._entries[key] = true;
                LoadEventData(out str2, out str3, entry.Descriptors);
                this._listItems.Add(new EpgEntry(serviceIdentifier, str2, str3, entry.StartTime.ToLocalTime(), entry.Duration));
            }
        }

        private static void LoadEventData(out string name, out string description, IEnumerable<Descriptor> descriptors)
        {
            ShortEvent event2 = null;
            var list = new List<ExtendedEvent>();
            foreach (var descriptor in descriptors)
            {
                if (descriptor.IsValid)
                {
                    if (event2 == null)
                    {
                        event2 = descriptor as ShortEvent;
                        if (event2 != null)
                        {
                            continue;
                        }
                    }
                    var item = descriptor as ExtendedEvent;
                    if (item != null)
                    {
                        list.Add(item);
                    }
                }
            }
            description = null;
            name = null;
            if (list.Count > 0)
            {
                var builder = new StringBuilder();
                foreach (var event4 in list)
                {
                    if (name == null)
                    {
                        name = event4.Name;
                    }
                    if (event4.Text != null)
                    {
                        builder.Append(event4.Text);
                    }
                }
                description = builder.ToString();
            }
            if (event2 != null)
            {
                if (name == null)
                {
                    name = event2.Name;
                }
                if (description == null)
                {
                    description = event2.Text;
                }
                if (string.IsNullOrEmpty(name))
                {
                    name = event2.Text;
                }
                else if (!string.IsNullOrEmpty(event2.Text))
                {
                    name = name + string.Format(" ({0})", event2.Text);
                }
            }
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            var dlg = new SelectSourceDialog();
            if (DialogResult.OK == dlg.ShowDialog())
            {                 
                btnStart.Enabled = true;
                btnStop.Enabled = true;
                _streamUri =dlg.Uri;                        
                toolStripStatusLabel1.Text = dlg.Uri.ToString();                
            }
        }
        
        private void btnStart_Click(object sender, EventArgs e)
        {
            _client.Start(_streamUri);           
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            _client.Stop();
            timer1.Enabled = true;            
        }
        private void lstEntries_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            int sortIndex = this.SortIndex;
            if (sortIndex <= 0)
            {
                sortIndex = -sortIndex;
            }
            int num2 = e.Column + 1;
            if (num2 == sortIndex)
            {
                this.SortIndex = -this.SortIndex;
            }
            else
            {
                this.SortIndex = num2;
            }
            this.listView1.Sort();
        }

        private void lstEntries_DoubleClick(object sender, EventArgs e)
        {
            if (1 == this.listView1.SelectedItems.Count)
            {
                EpgEntry entry = this.listView1.SelectedItems[0] as EpgEntry;
                if (entry != null)
                {
                    using (EpgDisplay display = new EpgDisplay(entry))
                    {
                        if (DialogResult.OK == display.ShowDialog(this))
                        {
                            base.Close();
                        }
                    }
                }
            }
        }
        private void timer1_Tick(object sender, EventArgs e)
        {
            this.timer1.Enabled = false;            
            bool loading = true;            
            try
            {                
                bool flag = true;
                byte[] buffer = new byte[flag ? 0x989680 : 0x186a0];
                using (MemoryStream stream = new MemoryStream(_client.Payload))
                {
                    using (TSParser parser = new TSParser())
                    {
                        int num;
                        if (flag)
                        {
                            parser.SetFilter(0x12, true, new FilterHandler(this.EITParser.OnData));                            
                        }
                        while ((num = stream.Read(buffer, 0, buffer.Length)) > 0)
                        {
                            
                            Application.DoEvents();
                            if (!this.btnStop.Enabled)
                            {
                                goto Label_015C;
                            }
                            if (flag)
                            {
                                parser.AddPayload(buffer, 0, num);
                            }
                        }
                    }
                }
            }
            catch (Exception exception)
            {
                MessageBox.Show(this, exception.Message);
            }
            finally
            {
                loading = false;
            }
        Label_015C:            
            this.btnStop.Enabled = true;            
            foreach (EpgEntry item in _listItems)
            {                 
                if (!listView1.Items.Contains(item))
                {
                    listView1.Items.Add(item);
                }                
            }
            this.listView1.ListViewItemSorter = new EpgEntry.Comparer();
        }       
    }
}
